from collections import defaultdict

from metadocker.cmdline.commands.base import Argument, Command
from metadocker.common.errors import SimpleError
from metadocker.common.misc import aligned_table
from metadocker.db.db import settings
from metadocker.db.manager import DockerManager
from metadocker.register import load_env


class ListCommand(Command):
    NAME = "list"
    HELP = "Liste les scripts"
    ARGUMENTS = [
        Argument("--short", "-s", action="store_true", help="Afficher simplement la liste"),
        Argument("--long", "-l", action="store_true", help="Affichage long"),
        Argument("--fields", "-f",
                 help="La liste des champs à afficher séparé par des virgules",
                 default="name,PROJECT,DOCKER_NAME,PORTS,VOLUMES,running")
    ]

    def run(self):
        manager : DockerManager = settings.app_docker_cache
        manager.refresh()

        def _getattr(docker, k):
            return getattr(docker.meta, k,
                            getattr(docker, k, ""))


        fields = self.fields.split(",")
        if self.short:
            print(f"Dockers disponibles: ", "\n" if manager.dockers else "", "\n  ".join(manager.dockers))
        elif  self.long:
            content=[]
            for name, docker in manager.dockers.items():
                dock = []
                for k in fields:
                    dock.append(f"{k}={_getattr(docker, k)}")
                content.append("\n".join(dock))
            if content:
                print("\n\n".join(content))
        else:
            table = [fields]
            for name in sorted(manager.dockers):
                line = []
                dock = manager.dockers[name]
                for k in fields:
                    line.append(_getattr(dock, k))
                table.append(line)
            print(aligned_table(table))

