import inspect
import os
import sys
from collections import defaultdict
from pathlib import Path


def get_root_dir():
    return Path(sys.argv[0]).parent.resolve()


def flatten(input, out=None):
    out = out if out is not None else []
    if not isinstance(input, (list, tuple, set)):
        input = [input]
    if isinstance(out, (tuple, set)):
        out = list(out)
    for x in input:
        if isinstance(x, (list, tuple, set)):
            flatten(x, out)
        else:
            out.append(x)
    return out

def aligned_table(table, sep="\t"):
    sizes = defaultdict(int)
    for line in table:
        for i, val in list(enumerate(line)):
            val = str(val)
            line[i] = val
            sizes[i] = max(sizes[i], len(val))

    for line in table:
        for i, val in enumerate(line):
            line[i] = val.ljust(sizes[i], " ")
    content = "\n".join([
        sep.join(
            val.ljust(sizes[i], " ")
            for i, val in enumerate(line)
        ) for line in table
    ])
    return content
